@@ -1,9 +1,9 @@
 /*
- * "$Id: socket.c,v 1.9.2.2 2002/12/05 22:56:05 jlovell Exp $"
+ * "$Id: socket.c,v 1.17 2005/01/04 22:10:36 jlovell Exp $"
  *
  *   AppSocket backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -41,11 +41,13 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <stdarg.h>
+#include <cups/http-private.h>
 #include <cups/string.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <signal.h>
+#include <sys/param.h>		/* for MAX() definition */
 
 #ifdef WIN32
 #  include <winsock.h>
@@ -58,8 +60,8 @@
 #  include <netdb.h>
 #endif /* WIN32 */
 
-static int printFile(int fdin, int fileOut, int socket);
-static void getSocketOptions(const char *uri, int *bidiP);
+static int printFile(int fdin, int fileOut, int socket, int waitEOF);
+static void getSocketOptions(const char *uri, int *bidiP, int *waitEOFP);
 static int sendUrgentReset(int socket);
 static void sighup_handler(int sig);
 
@@ -69,6 +71,13 @@ static void sighup_handler(int sig);
  */
 static int gSocketOut = -1;
 
+/*
+ * Local functions...
+ */
+
+void	print_backchannel(const unsigned char *buffer, size_t nbytes);
+
+
 /*
  * 'main()' - Send a file to the printer or server.
  *
@@ -94,6 +103,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   struct sockaddr_in addr;	/* Socket address */
   struct hostent *hostaddr;	/* Host address */
   int           bidi;		/* If true send the backchannel data to stdout else stderr. */
+  int           waitEOF;	/* If true wait for eof to be sent. */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction action;	/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
@@ -105,21 +115,28 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   setbuf(stderr, NULL);
 
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGHUP, sighup_handler);
+ /*
+  * Ignore SIGPIPE and catch SIGHUP signals...
+  */
 
+#ifdef HAVE_SIGSET
+  sigset(SIGPIPE, SIG_IGN);
+  sigset(SIGHUP, sighup_handler);
 #elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &action, NULL);
+
   memset(&action, 0, sizeof(action));
   sigemptyset(&action.sa_mask);
   sigaddset(&action.sa_mask, SIGHUP);
   action.sa_handler = sighup_handler;
   sigaction(SIGHUP, &action, NULL);
-  
 #else
+  signal(SIGPIPE, SIG_IGN);
   signal(SIGHUP, sighup_handler);
 #endif /* HAVE_SIGSET */
 
-
  /*
   * Check command-line...
   */
@@ -177,7 +194,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if ((hostaddr = httpGetHostByName(hostname)) == NULL)
   {
     fprintf(stderr, "ERROR: Unable to locate printer \'%s\' - %s\n",
-            hostname, strerror(errno));
+            hostname, hstrerror(h_errno));
     return (1);
   }
 
@@ -266,11 +283,11 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
      */
     gSocketOut = fd;
     
-    getSocketOptions(argv[0], &bidi);
+    getSocketOptions(argv[0], &bidi, &waitEOF);
 
     if (bidi) fprintf(stderr, "DEBUG: socket bidirectional enabled.\n");
 
-    (void) printFile(fp, bidi ? STDOUT_FILENO : -1, fd);
+    (void) printFile(fp, bidi ? STDOUT_FILENO : -1, fd, waitEOF);
     
     /* We're done with the socket so we'll clear the global
      * used by the HUP signal handler.
@@ -296,7 +313,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   return (0);
 }
 
-static int printFile(int fdin, int fileOut, int socket)
+static int printFile(int fdin, int fileOut, int socket, int waitEOF)
 {
   char fileBuffer[16 * 1024];
   char socketReadBuffer[1 * 1024];
@@ -393,7 +410,8 @@ static int printFile(int fdin, int fileOut, int socket)
            * shutdown the socket and wait for the other end to finish...
            */
           fputs("INFO: Print file sent, waiting for printer to finish...\n", stderr);
-          shutdown(socket, 1);
+          if (!waitEOF)
+            shutdown(socket, 1);
         } else {
           perror("ERROR: socket reading from input steam");
           err = errno;
@@ -482,24 +500,28 @@ static int printFile(int fdin, int fileOut, int socket)
       
   }
 
+  if (waitEOF)
+    shutdown(socket, 1);
+
   fprintf(stderr, "DEBUG: socket finished sending file, err = %d.\n", err);
 
   return err;
 }
 
-static void getSocketOptions(const char *uri, int *bidiP)
+static void getSocketOptions(const char *uri, int *bidiP, int *waitEOFP)
 {
   char method[255];	/* Method in URI */
   char hostname[1024];	/* Hostname */
   char username[255];	/* Username info (not used) */
   char resource[1024];	/* Resource info (device and options) */
   int port = 0;		/* Port number (not used) */
-  char *resourcePtr = NULL;
   char *options = NULL;	/* Pointer to options */
   char optionName[255];	/* Name of option */
   char value[255];	/* Value of option */
   char *ptr = NULL;     /* Pointer into name or value */
-  int bidi = false;
+
+  /* Initialize the arguments passed in */
+  *bidiP = *waitEOFP = false;
 
   /* Extract the device name and options from the URI...
    */
@@ -524,10 +546,11 @@ static void getSocketOptions(const char *uri, int *bidiP)
       /*
       * Get the name...
       */
-      for (ptr = optionName; *options && *options != '=';)
+      for (ptr = optionName; *options && *options != '=' && *options != '+'; )
         *ptr++ = *options++;
 
       *ptr = '\0';
+      *value = '\0';
                 
       if (*options == '=')
       {
@@ -545,20 +568,27 @@ static void getSocketOptions(const char *uri, int *bidiP)
           options ++;
           
       }
-      else
+      else if (*options == '+')
       {
-        value[0] = '\0';
+        options ++;
       }
-      
+
       if (strcasecmp(optionName, "bidi") == 0)
       {
-        bidi = true;
+        *bidiP = *value == '\0' || 
+                 strcasecmp(value, "on") == 0  ||
+                 strcasecmp(value, "yes") == 0 ||
+                 strcasecmp(value, "true") == 0;
+      }
+      else if (strcasecmp(optionName, "waiteof") == 0)
+      {
+        *waitEOFP = *value == '\0' || 
+                    strcasecmp(value, "on") == 0  ||
+                    strcasecmp(value, "yes") == 0 || 
+                    strcasecmp(value, "true") == 0;
       }
     } 
   }
-  
-  if (bidiP) *bidiP = bidi;
-  
 }
 
 static int sendUrgentReset(int socket)
@@ -598,5 +628,43 @@ static void sighup_handler(int sig)
 
 
 /*
- * End of "$Id: socket.c,v 1.9.2.2 2002/12/05 22:56:05 jlovell Exp $".
+ * 'print_backchannel()' - Print the contents of a back-channel buffer.
+ */
+
+void
+print_backchannel(const unsigned char *buffer,	/* I - Data buffer */
+                  size_t              nbytes)	/* I - Number of bytes */
+{
+  char	line[255],				/* Formatted line */
+	*lineptr;				/* Pointer into line */
+
+
+  for (lineptr = line; nbytes > 0; buffer ++, nbytes --)
+  {
+    if (*buffer < 0x20 || *buffer >= 0x7f)
+    {
+      snprintf(lineptr, sizeof(line) - (lineptr - line), "<%02X>", *buffer);
+      lineptr += strlen(lineptr);
+    }
+    else
+      *lineptr++ = *buffer;
+
+    if ((lineptr - line) > 72)
+    {
+      *lineptr = '\0';
+      fprintf(stderr, "DEBUG: DATA: %s\n", line);
+      lineptr = line;
+    }
+  }
+
+  if (lineptr > line)
+  {
+    *lineptr = '\0';
+    fprintf(stderr, "DEBUG: DATA: %s\n", line);
+  }
+}
+
+
+/*
+ * End of "$Id: socket.c,v 1.17 2005/01/04 22:10:36 jlovell Exp $".
  */
