@@ -1,12 +1,33 @@
 /*
- * USB printer backend for CUPS.
+ * "$Id: usb.c,v 1.1.1.18.2.1 2005/09/27 23:23:42 jlovell Exp $"
  *
- * Copyright © 2020-2024 by OpenPrinting.
- * Copyright © 2007-2012 by Apple Inc.
- * Copyright © 1997-2007 by Easy Software Products, all rights reserved.
+ *   USB port backend for the Common UNIX Printing System (CUPS).
  *
- * Licensed under Apache License v2.0.  See the file "LICENSE" for more
- * information.
+ *   Copyright 1997-2005 by Easy Software Products, all rights reserved.
+ *
+ *   These coded instructions, statements, and computer programs are the
+ *   property of Easy Software Products and are protected by Federal
+ *   copyright law.  Distribution and use rights are outlined in the file
+ *   "LICENSE" which should have been included with this file.  If this
+ *   file is missing or damaged please contact Easy Software Products
+ *   at:
+ *
+ *       Attn: CUPS Licensing Information
+ *       Easy Software Products
+ *       44141 Airport View Drive, Suite 204
+ *       Hollywood, Maryland 20636 USA
+ *
+ *       Voice: (301) 373-9600
+ *       EMail: cups-info@cups.org
+ *         WWW: http://www.cups.org
+ *
+ *   This file is subject to the Apple OS-Developed Software exception.
+ *
+ * Contents:
+ *
+ *   list_devices() - List all available USB devices to stdout.
+ *   print_device() - Print a file to a USB device.
+ *   main()         - Send a file to the specified USB port.
  */
 
 /*
@@ -18,15 +39,20 @@
 #  include <ApplicationServices/ApplicationServices.h>
 #endif /* __APPLE__ */
 
-#include "backend-private.h"
+#include <cups/cups.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <errno.h>
+#include <cups/string.h>
+#include <signal.h>
 
-#ifdef _WIN32
+#ifdef WIN32
 #  include <io.h>
 #else
 #  include <unistd.h>
 #  include <fcntl.h>
 #  include <termios.h>
-#endif /* _WIN32 */
+#endif /* WIN32 */
 
 
 /*
@@ -35,19 +61,17 @@
 
 void	list_devices(void);
 int	print_device(const char *uri, const char *hostname,
-	             const char *resource, char *options,
-		     int print_fd, int copies, int argc, char *argv[]);
+	             const char *resource, const char *options,
+		     int fp, int copies, int argc, char *argv[]);
 
 
 /*
  * Include the vendor-specific USB implementation...
  */
 
-#ifdef HAVE_LIBUSB
-#  include "usb-libusb.c"
-#elif defined(__APPLE__)
+#ifdef __APPLE__
 #  include "usb-darwin.c"
-#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__DragonFly__) || defined(__FreeBSD_kernel__)
+#elif defined(__linux) || defined(__sun) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #  include "usb-unix.c"
 #else
 /*
@@ -82,11 +106,9 @@ int					/* O - Exit status */
 print_device(const char *uri,		/* I - Device URI */
              const char *hostname,	/* I - Hostname/manufacturer */
              const char *resource,	/* I - Resource/modelname */
-	     char       *options,	/* I - Device options/serial number */
-	     int        print_fd,	/* I - File descriptor to print */
-	     int        copies,		/* I - Copies to print */
-	     int	argc,		/* I - Number of command-line arguments (6 or 7) */
-	     char	*argv[])	/* I - Command-line arguments */
+	     const char *options,	/* I - Device options/serial number */
+	     int        fp,		/* I - File descriptor to print */
+	     int        copies)		/* I - Copies to print */
 {
  /*
   * Can't print, so just reference the arguments to eliminate compiler
@@ -99,14 +121,12 @@ print_device(const char *uri,		/* I - Device URI */
   (void)hostname;
   (void)resource;
   (void)options;
-  (void)print_fd;
+  (void)fp;
   (void)copies;
-  (void)argc;
-  (void)argv;
 
-  return (CUPS_BACKEND_FAILED);
+  return (1);
 }
-#endif /* HAVE_LIBUSB */
+#endif /* __APPLE__ */
 
 
 /*
@@ -121,7 +141,7 @@ int					/* O - Exit status */
 main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
      char *argv[])			/* I - Command-line arguments */
 {
-  int		print_fd;		/* Print file */
+  int		fp;			/* Print file */
   int		copies;			/* Number of copies to print */
   int		status;			/* Exit status */
   int		port;			/* Port number (not used) */
@@ -163,33 +183,32 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   if (argc == 1)
   {
     list_devices();
-    return (CUPS_BACKEND_OK);
+    return (0);
   }
-  else if (argc != 6 && argc != 7)
+  else if (argc < 6 || argc > 7)
   {
-    _cupsLangPrintf(stderr,
-                    _("Usage: %s job-id user title copies options [file]"),
-                    argv[0]);
-    return (CUPS_BACKEND_FAILED);
+    fputs("Usage: usb job-id user title copies options [file]\n", stderr);
+    return (1);
   }
 
  /*
   * Extract the device name and options from the URI...
   */
 
-  uri = cupsBackendDeviceURI(argv);
+  if (strncmp(argv[0], "usb:", 4))
+    uri = getenv("DEVICE_URI");
+  else
+    uri = argv[0];
 
-  if (httpSeparateURI(HTTP_URI_CODING_ALL, uri,
-                      method, sizeof(method), username, sizeof(username),
-		      hostname, sizeof(hostname), &port,
-		      resource, sizeof(resource)) < HTTP_URI_OK)
+  if (!uri)
   {
-    _cupsLangPrintFilter(stderr, "ERROR",
-			 _("No device URI found in argv[0] or in DEVICE_URI "
-                           "environment variable."));
+    fputs("ERROR: No device URI found in argv[0] or in DEVICE_URI environment variable!\n", stderr);
     return (1);
   }
 
+  /* Use uri instead of argv[0] to get around radr://4212077 for now... */
+  httpSeparate(uri, method, username, hostname, &port, resource);
+
  /*
   * See if there are any options...
   */
@@ -211,8 +230,8 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
 
   if (argc == 6)
   {
-    print_fd = 0;
-    copies   = 1;
+    fp     = 0;
+    copies = 1;
   }
   else
   {
@@ -220,10 +239,11 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
     * Try to open the print file...
     */
 
-    if ((print_fd = open(argv[6], O_RDONLY)) < 0)
+    if ((fp = open(argv[6], O_RDONLY)) < 0)
     {
-      _cupsLangPrintError("ERROR", _("Unable to open print file"));
-      return (CUPS_BACKEND_FAILED);
+      fprintf(stderr, "ERROR: unable to open print file %s - %s\n",
+              argv[6], strerror(errno));
+      return (1);
     }
 
     copies = atoi(argv[4]);
@@ -233,15 +253,19 @@ main(int  argc,				/* I - Number of command-line arguments (6 or 7) */
   * Finally, send the print file...
   */
 
-  status = print_device(uri, hostname, resource, options, print_fd, copies,
-                        argc, argv);
+  status = print_device(uri, hostname, resource, options, fp, copies, argc, argv);
 
  /*
   * Close the input file and return...
   */
 
-  if (print_fd != 0)
-    close(print_fd);
+  if (fp != 0)
+    close(fp);
 
   return (status);
 }
+
+
+/*
+ * End of "$Id: usb.c,v 1.1.1.18.2.1 2005/09/27 23:23:42 jlovell Exp $".
+ */
