@@ -1,9 +1,9 @@
 /*
- * "$Id: common.h,v 1.2 2002/05/28 23:20:45 gelphman Exp $"
+ * "$Id: common.h,v 1.1.1.12 2005/01/04 19:15:55 jlovell Exp $"
  *
  *   Common filter definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -40,6 +40,15 @@
 #include <cups/string.h>
 
 
+/*
+ * C++ magic...
+ */
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
+
 /*
  * Globals...
  */
@@ -70,5 +79,14 @@ extern void	WriteLabels(int orient);
 
 
 /*
- * End of "$Id: common.h,v 1.2 2002/05/28 23:20:45 gelphman Exp $".
+ * C++ magic...
+ */
+
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+
+/*
+ * End of "$Id: common.h,v 1.1.1.12 2005/01/04 19:15:55 jlovell Exp $".
  */
