@@ -1,9 +1,9 @@
 /*
- * "$Id: image-png.c,v 1.1.1.4.2.1 2002/12/13 22:54:10 jlovell Exp $"
+ * "$Id: image-png.c,v 1.1.1.12 2005/01/04 19:15:59 jlovell Exp $"
  *
  *   PNG image routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2005 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -15,9 +15,9 @@
  *       Attn: CUPS Licensing Information
  *       Easy Software Products
  *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
+ *       Hollywood, Maryland 20636 USA
  *
- *       Voice: (301) 373-9603
+ *       Voice: (301) 373-9600
  *       EMail: cups-info@cups.org
  *         WWW: http://www.cups.org
  *
@@ -62,6 +62,7 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 		*out;			/* Output pixels */
   png_color_16	bg;			/* Background color */
 
+
  /*
   * Setup the PNG data structures...
   */
@@ -81,14 +82,26 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 
   png_read_info(pp, info);
 
-  if (info->color_type == PNG_COLOR_TYPE_PALETTE)
-    png_set_expand(pp);
+  fprintf(stderr, "DEBUG: PNG image: %dx%dx%d, color_type=%x (%s%s%s)\n",
+          info->width, info->height, info->bit_depth, info->color_type,
+	  (info->color_type & PNG_COLOR_MASK_COLOR) ? "RGB" : "GRAYSCALE",
+	  (info->color_type & PNG_COLOR_MASK_ALPHA) ? "+ALPHA" : "",
+	  (info->color_type & PNG_COLOR_MASK_PALETTE) ? "+PALETTE" : "");
 
-  if (info->color_type == PNG_COLOR_TYPE_GRAY ||
-      info->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-    img->colorspace = secondary;
-  else
+  if (info->color_type & PNG_COLOR_MASK_PALETTE)
+    png_set_expand(pp);
+  else if (info->bit_depth < 8)
+  {
+    png_set_packing(pp);
+    png_set_expand(pp);
+  }
+  else if (info->bit_depth == 16)
+    png_set_strip_16(pp);
+
+  if (info->color_type & PNG_COLOR_MASK_COLOR)
     img->colorspace = (primary == IMAGE_RGB_CMYK) ? IMAGE_RGB : primary;
+  else
+    img->colorspace = secondary;
 
   if (info->width == 0 || info->width > IMAGE_MAX_WIDTH ||
       info->height == 0 || info->height > IMAGE_MAX_HEIGHT)
@@ -119,16 +132,6 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 
   ImageSetMaxTiles(img, 0);
 
-  if (info->bit_depth < 8)
-  {
-    png_set_packing(pp);
-
-    if (info->valid & PNG_INFO_sBIT)
-      png_set_shift(pp, &(info->sig_bit));
-  }
-  else if (info->bit_depth == 16)
-    png_set_strip_16(pp);
-
   passes = png_set_interlace_handling(pp);
 
  /*
@@ -187,29 +190,7 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
         * Output this row...
 	*/
 
-	if (info->color_type == PNG_COLOR_TYPE_GRAY ||
-	     info->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-	{
-	  switch (img->colorspace)
-	  {
-	    case IMAGE_WHITE :
-		memcpy(out, inptr, img->xsize);
-		break;
-	    case IMAGE_RGB :
-		ImageWhiteToRGB(inptr, out, img->xsize);
-		break;
-	    case IMAGE_BLACK :
-		ImageWhiteToBlack(inptr, out, img->xsize);
-		break;
-	    case IMAGE_CMY :
-		ImageWhiteToCMY(inptr, out, img->xsize);
-		break;
-	    case IMAGE_CMYK :
-		ImageWhiteToCMYK(inptr, out, img->xsize);
-		break;
-	  }
-	}
-	else
+	if (info->color_type & PNG_COLOR_MASK_COLOR)
 	{
 	  if ((saturation != 100 || hue != 0) && bpp > 1)
 	    ImageRGBAdjust(inptr, img->xsize, saturation, hue);
@@ -233,6 +214,27 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 		break;
 	  }
 	}
+	else
+	{
+	  switch (img->colorspace)
+	  {
+	    case IMAGE_WHITE :
+		memcpy(out, inptr, img->xsize);
+		break;
+	    case IMAGE_RGB :
+		ImageWhiteToRGB(inptr, out, img->xsize);
+		break;
+	    case IMAGE_BLACK :
+		ImageWhiteToBlack(inptr, out, img->xsize);
+		break;
+	    case IMAGE_CMY :
+		ImageWhiteToCMY(inptr, out, img->xsize);
+		break;
+	    case IMAGE_CMYK :
+		ImageWhiteToCMYK(inptr, out, img->xsize);
+		break;
+	  }
+	}
 
 	if (lut)
 	  ImageLut(out, img->xsize * bpp, lut);
@@ -242,11 +244,10 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 
       if (passes > 1)
       {
-	if (info->color_type == PNG_COLOR_TYPE_GRAY ||
-		 info->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-          inptr += img->xsize;
-	else
+	if (info->color_type & PNG_COLOR_MASK_COLOR)
           inptr += img->xsize * 3;
+	else
+          inptr += img->xsize;
       }
     }
 
@@ -265,5 +266,5 @@ ImageReadPNG(image_t    *img,		/* IO - Image */
 
 
 /*
- * End of "$Id: image-png.c,v 1.1.1.4.2.1 2002/12/13 22:54:10 jlovell Exp $".
+ * End of "$Id: image-png.c,v 1.1.1.12 2005/01/04 19:15:59 jlovell Exp $".
  */
